<?php
/**
 * Generated by PHPUnit_SkeletonGenerator on 2014-11-20 at 06:53:40.
 */
namespace chippyash\Test\BuilderPattern;

use chippyash\BuilderPattern\AbstractDirector;
use chippyash\BuilderPattern\BuilderModifer;

class AbstractDirectorTest extends \PHPUnit_Framework_TestCase
{

    /**
     * Mock
     * @var AbstractDirector
     */
    protected $object;
    /**
     * Mock
     * @var chippyash\BuilderPattern\BuilderInterface 
     */
    protected $builder;
    /**
     * Mock
     * @var chippyash\BuilderPattern\RendererInterface
     */
    protected $renderer;
    
    protected $modifer;
    
    protected function setUp()
    {
        $this->builder = $this->getMock('chippyash\BuilderPattern\BuilderInterface');
        $this->renderer = $this->getMock('chippyash\BuilderPattern\RendererInterface');
        $this->object = $this->getMockForAbstractClass(
                'chippyash\BuilderPattern\AbstractDirector',
                [$this->builder, $this->renderer]);
    }

    public function testBuildWillReturnValueIfBuildSucceeds()
    {
        $test = ['foo' => 'bar'];
        $this->builder->expects($this->once())
                ->method('build')
                ->will($this->returnValue(true));
        $this->renderer->expects($this->once())
                ->method('render')
                ->will($this->returnValue($test));
        
        $this->assertEquals($test, $this->object->build());
    }

    /**
     * @expectedException chippyash\BuilderPattern\Exceptions\BuilderPatternException
     * @expectedExceptionMessage Unable to build
     */
    public function testBuildWillThrowExceptionIfBuildFails()
    {
        $this->builder->expects($this->once())
                ->method('build')
                ->will($this->returnValue(false));
        
        $this->object->build();
    }
    
    public function testSettingAModifierWillSetTheModifier()
    {
        $modifier = $this->getMock('Zend\EventManager\EventManagerAwareInterface');
        $this->object->setModifier($modifier);
        $refl = new \ReflectionObject($this->object);
        $prop = $refl->getProperty('modifier');
        $prop->setAccessible(true);
        
        $this->assertEquals($prop->getValue($this->object), $modifier);
    }
    
    public function testSettingAModifierWillPropagateToBuildersIfTheyAreModifiable()
    {
        $this->markTestIncomplete('create test when builders are modifiable');
    }
    
    public function testCallingModifyWillReturnAnEmptyZendEventResponseCollectionIfModifierNotSet()
    {
        $test = $this->object->modify();
        $this->assertInstanceOf(
                'Zend\EventManager\ResponseCollection', 
                $test);
        $this->assertEquals(0, $test->count());
    }
    
    public function testCallingModifyWillReturnAZendEventResponseCollectionIfModifierSet()
    {
        $modifier = $this->getMock('Zend\EventManager\EventManagerAwareInterface');
        $modifier->expects($this->once())
                ->method('getEventManager')
                ->willReturn(new \Zend\EventManager\EventManager());
        $this->object->setModifier($modifier);
        $test = $this->object->modify();
        $this->assertInstanceOf(
                'Zend\EventManager\ResponseCollection', 
                $test);
        $this->assertEquals(0, $test->count());
    }
}
